% ASLogger data processing.

close all; clear all;

% Remove the header (if any) indicating the meaning and units of each
% column in the data files, otherwise impordata function might fail (check
% also if it is not repeated in the middle of the file, this can happen if
% the Android application was restarted). Remove also any invalid GPS data
% in the files (usually lines in the beginning with lots of 0).

% Rear smartphone.
log0=importdata('log0.txt');
nanoTime0=log0(:,1);
utcMilliTime0=log0(:,2);
lat0=log0(:,3);
long0=log0(:,4);
speed0=log0(:,6);
bearing0=log0(:,7);
azimuth0=log0(:,8);
pitch0=log0(:,9);
roll0=log0(:,10);
n0=size(nanoTime0); n0=n0(1);

% Front smartphone.
log1=importdata('log1.txt');
nanoTime1=log1(:,1);
utcMilliTime1=log1(:,2);
lat1=log1(:,3);
long1=log1(:,4);
speed1=log1(:,6);
bearing1=log1(:,7);
azimuth1=log1(:,8);
pitch1=log1(:,9);
roll1=log1(:,10);
n1=size(nanoTime1); n1=n1(1);

% Convert times in the same unit (ms) and with the same time origin (the one of utcMilliTime).
% Take the first systemNanoTime that corresponds to the last utcMilliTime...
for i0=n0:-1:1,
    if (utcMilliTime0(i0) < utcMilliTime0(n0))
        if (i0+1 <= n0)
            t0=int64((nanoTime0+(utcMilliTime0(n0)*1000000-nanoTime0(i0+1)))/1000000.0);
            int64(nanoTime0(i0+1)) % Debug...
        else
            t0=int64((nanoTime0+(utcMilliTime0(n0)*1000000-nanoTime0(n0)))/1000000.0);
            int64(nanoTime0(n0)) % Debug...
        end
        break
    end
end
for i1=n1:-1:1,
    if (utcMilliTime1(i1) < utcMilliTime1(n1))
        if (i1+1 <= n1)
            t1=int64((nanoTime1+(utcMilliTime1(n1)*1000000-nanoTime1(i1+1)))/1000000.0);
            int64(nanoTime1(i1+1)) % Debug...
        else
            t1=int64((nanoTime1+(utcMilliTime1(n1)*1000000-nanoTime1(n1)))/1000000.0);
            int64(nanoTime1(n1)) % Debug...
        end
        break
    end
end

% Gather all time values found in the data.
t=sort([t0;t1]);
n=size(t); n=n(1);

% Time synchronizations and conversions in a local East-North-Up coordinate
% system with (lat1(1);long1(1)) as origin, and angles in rad.
lat=zeros(n,1,'double');
long=zeros(n,1,'double');
x=zeros(n,1,'double');
y=zeros(n,1,'double');
theta=zeros(n,1,'double');
thetaplusdelta=zeros(n,1,'double');
delta=zeros(n,1,'double');
phi=zeros(n,1,'double');
v=zeros(n,1,'double');
bearing=zeros(n,1,'double');
curj0=1;
curj1=1;
for i=1:n,
    for j0=curj0:n0,
        if (t0(j0) >= t(i))
            theta(i)=fmod_2PI(pi/2.0-azimuth0(j0)*pi/180.0);
            phi(i)=fmod_2PI(-roll0(j0)*pi/180.0);
            bearing(i)=fmod_2PI(pi/2.0-bearing0(j0)*pi/180.0);
            curj0=j0;
            break
        end
    end
    for j1=curj1:n1,
        if (t1(j1) >= t(i))
            lat(i)=lat1(j1);
            long(i)=long1(j1);
            v(i)=speed1(j1);
            [x(i),y(i)]=GPS2RefCoordSystem(lat1(1), long1(1), lat(i), long(i));
            thetaplusdelta(i)=fmod_2PI(pi/2.0-azimuth1(j1)*pi/180.0);
            curj1=j1;
            break
        end
    end
end

for i=1:n,
    delta(i)=fmod_2PI(thetaplusdelta(i)-theta(i));
end

% Gather all variables correctly synchronized.
alldata=[double(t) lat long x y theta thetaplusdelta delta phi v bearing];
dlmwrite('alldata.txt', alldata, 'delimiter', ';', 'precision', '%16.8f');

figure('Position',[100 100 400 400]);
clf;
hold on;
axis([-150,150,-150,150]);
plot(x,y);
%figure('Name','theta','NumberTitle','off');
%plot(theta);
%figure('Name','thetaplusdelta','NumberTitle','off');
%plot(thetaplusdelta);
%figure('Name','err','NumberTitle','off');
%plot(fmod_2PI(bearing-theta))
figure('Name','v','NumberTitle','off');
plot(v)
figure('Name','delta','NumberTitle','off');
plot(delta);
figure('Name','phi','NumberTitle','off');
plot(phi,'Color',[1 0 0]);
