% ASLogger data processing.

close all; clear all;

% Remove the header (if any) indicating the meaning and units of each
% column in the data files, otherwise impordata function might fail (check
% also if it is not repeated in the middle of the file, this can happen if
% the Android application was restarted). Remove also any invalid GPS data
% in the files (usually lines in the beginning with lots of 0).

log0=importdata('log.txt');
n00=size(log0); n00=n00(1);
nanoTime0=log0(1:4:n00,1);
utcMilliTime0=log0(1:4:n00,2);
lat0=log0(1:4:n00,3);
long0=log0(1:4:n00,4);
alt0=log0(1:4:n00,5);
speed0=log0(1:4:n00,6);
bearing0=log0(1:4:n00,7);
azimuth0=log0(1:4:n00,8);
pitch0=log0(1:4:n00,9);
roll0=log0(1:4:n00,10);
n0=size(nanoTime0); n0=n0(1);

log1=importdata('log_mti.txt');
n01=size(log1); n01=n01(1);
roll1=log1(1:25:n01,1);
pitch1=log1(1:25:n01,2);
yaw1=log1(1:25:n01,3);
n1=size(roll1); n1=n1(1);

% File creation time, file modification time...
utcMilliTime1_begin=1398171229000; utcMilliTime1_end=1398174902000;
t1=linspace(utcMilliTime1_begin,utcMilliTime1_end,n1)';

% Convert times in the same unit (ms) and with the same time origin (the one of utcMilliTime).
% Take the first systemNanoTime that corresponds to the last utcMilliTime...
for i0=n0:-1:1,
    if (utcMilliTime0(i0) < utcMilliTime0(n0))
        if (i0+1 <= n0)
            t0=int64((nanoTime0+(utcMilliTime0(n0)*1000000-nanoTime0(i0+1)))/1000000.0);
            int64(nanoTime0(i0+1)) % Debug...
        else
            t0=int64((nanoTime0+(utcMilliTime0(n0)*1000000-nanoTime0(n0)))/1000000.0);
            int64(nanoTime0(n0)) % Debug...
        end
        break
    end
end

% Gather all time values found in the data.
t=sort([t0;t1]);
n=size(t); n=n(1);

% Time synchronizations and conversions in a local East-North-Up coordinate
% system with (lat1(1);long1(1)) as origin, and angles in rad.
lat=zeros(n,1,'double');
long=zeros(n,1,'double');
alt=zeros(n,1,'double');
v=zeros(n,1,'double');
bearing=zeros(n,1,'double');
psi=zeros(n,1,'double');
theta=zeros(n,1,'double');
phi=zeros(n,1,'double');
x=zeros(n,1,'double');
y=zeros(n,1,'double');
z=zeros(n,1,'double');
psi_mti=zeros(n,1,'double');
theta_mti=zeros(n,1,'double');
phi_mti=zeros(n,1,'double');
curj0=1;
curj1=1;
for i=1:n,
    for j0=curj0:n0,
        if (t0(j0) >= t(i))
            lat(i)=lat0(j0);
            long(i)=long0(j0);
            alt(i)=alt0(j0);
            v(i)=speed0(j0);
            bearing(i)=fmod_2PI(pi/2.0-bearing0(j0)*pi/180.0);
            psi(i)=fmod_2PI(pi/2.0-azimuth0(j0)*pi/180.0);
            theta(i)=fmod_2PI(pitch0(j0)*pi/180.0);
            phi(i)=fmod_2PI(-roll0(j0)*pi/180.0);
            [x(i),y(i)]=GPS2RefCoordSystem(lat0(1), long0(1), lat(i), long(i));
            z(i)=alt(i)-alt0(1);
            curj0=j0;
            break
        end
    end
    for j1=curj1:n1,
        if (t1(j1) >= t(i))
            psi_mti(i)=fmod_2PI(yaw1(j1)*pi/180.0+pi/2.0);
            theta_mti(i)=fmod_2PI(pitch1(j1)*pi/180.0);
            phi_mti(i)=fmod_2PI(roll1(j1)*pi/180.0);
            curj1=j1;
            break
        end
    end
end


% Gather all variables correctly synchronized.
%alldata=[double(t) lat long alt v bearing psi theta phi x y z];
alldata=[double(t) lat long alt v bearing psi_mti theta_mti phi_mti x y z];
dlmwrite('alldata.txt', alldata, 'delimiter', ';', 'precision', '%16.8f');

figure('Position',[100 100 400 400]);
clf;
hold on;
axis([-25000,5000,-5000,25000]);
plot(x,y);
figure('Name','bearing','NumberTitle','off');
plot(bearing);
%figure('Name','err','NumberTitle','off');
%plot(fmod_2PI(bearing-psi))
figure('Name','v','NumberTitle','off');
plot(v)
figure('Name','psi','NumberTitle','off');
plot(psi,'Color',[1 0 0]);
hold on;
plot(psi_mti,'Color',[0.5 0 0]);
figure('Name','theta','NumberTitle','off');
plot(theta,'Color',[0 1 0]);
hold on;
plot(theta_mti,'Color',[0 0.5 0]);
figure('Name','phi','NumberTitle','off');
plot(phi,'Color',[0 0 1]);
hold on;
plot(phi_mti,'Color',[0 0 0.5]);
figure('Name','alt','NumberTitle','off');
plot(alt,'Color',[0 1 0]);
